#include "Main.h"

Canvas::Canvas(Vector2n Position, bool Centered)
	: Camera(0, 0),
	  CameraZ(1),
	  m_Position(Position),
	  m_Dimensions(0, 0),
	  m_Centered(Centered)
{
}

Canvas::~Canvas()
{
}

void Canvas::MoveView(uint8 Degree, double MoveAmount, double A[2])
{
	switch (Degree)
	{
	case 0:
		Camera.X() -= MoveAmount / CameraZ;
		break;
	case 1:
		Camera.Y() -= MoveAmount / CameraZ;
		break;
	case 2:
		{
			if (MoveAmount > 0)
			{
				CameraZ *= 1.5;
				Camera.X() += A[0] * 0.5 / CameraZ;
				Camera.Y() += A[1] * 0.5 / CameraZ;
			}
			else if (MoveAmount < 0)
			{
				Camera.X() -= A[0] * 0.5 / CameraZ;
				Camera.Y() -= A[1] * 0.5 / CameraZ;
				CameraZ /= 1.5;
			}
		}
		break;
	default:
		break;
	}
}

void Canvas::RenderBackground()
{
	glBindTexture(GL_TEXTURE_2D, 2);

	/*glMatrixMode(GL_TEXTURE_MATRIX);
	glLoadIdentity();
	glScaled(CameraZ, CameraZ, CameraZ);
	glTranslated(-CameraX, -CameraY, 0);
	glMatrixMode(GL_MODELVIEW);*/

	glColor3d(1, 1, 1);
	glBegin(GL_QUADS);
		/*glTexCoord2d(0 / 256.0, 0 / 256.0); glVertex2d(-0.5 * m_CanvasSize[0], -0.5 * m_CanvasSize[1]);
		glTexCoord2d(0 / 256.0, m_CanvasSize[1] / 256.0); glVertex2d(-0.5 * m_CanvasSize[0], 0.5 * m_CanvasSize[1]);
		glTexCoord2d(m_CanvasSize[0] / 256.0, m_CanvasSize[1] / 256.0); glVertex2d(0.5 * m_CanvasSize[0], 0.5 * m_CanvasSize[1]);
		glTexCoord2d(m_CanvasSize[0] / 256.0, 0 / 256.0); glVertex2d(0.5 * m_CanvasSize[0], -0.5 * m_CanvasSize[1]);*/
		/*glTexCoord2d(-0.5 * m_CanvasSize[0] / 256.0 - 0.5, -0.5 * m_CanvasSize[1] / 256.0 - 0.5); glVertex2d(-0.5 * m_CanvasSize[0], -0.5 * m_CanvasSize[1]);
		glTexCoord2d(-0.5 * m_CanvasSize[0] / 256.0 - 0.5, 0.5 * m_CanvasSize[1] / 256.0 - 0.5); glVertex2d(-0.5 * m_CanvasSize[0], 0.5 * m_CanvasSize[1]);
		glTexCoord2d(0.5 * m_CanvasSize[0] / 256.0 - 0.5, 0.5 * m_CanvasSize[1] / 256.0 - 0.5); glVertex2d(0.5 * m_CanvasSize[0], 0.5 * m_CanvasSize[1]);
		glTexCoord2d(0.5 * m_CanvasSize[0] / 256.0 - 0.5, -0.5 * m_CanvasSize[1] / 256.0 - 0.5); glVertex2d(0.5 * m_CanvasSize[0], -0.5 * m_CanvasSize[1]);*/
		glTexCoord2d((-0.5 * m_Dimensions[0] + Camera.X() * CameraZ) / 256.0 / CameraZ - 0.5, (-0.5 * m_Dimensions[1] + Camera.Y() * CameraZ) / 256.0 / CameraZ - 0.5); glVertex2i(m_Position[0], m_Position[1]);
		glTexCoord2d((-0.5 * m_Dimensions[0] + Camera.X() * CameraZ) / 256.0 / CameraZ - 0.5, (0.5 * m_Dimensions[1] + Camera.Y() * CameraZ) / 256.0 / CameraZ - 0.5); glVertex2i(m_Position[0], m_Position[1] + m_Dimensions[1]);
		glTexCoord2d((0.5 * m_Dimensions[0] + Camera.X() * CameraZ) / 256.0 / CameraZ - 0.5, (0.5 * m_Dimensions[1] + Camera.Y() * CameraZ) / 256.0 / CameraZ - 0.5); glVertex2i(m_Position[0] + m_Dimensions[0], m_Position[1] + m_Dimensions[1]);
		glTexCoord2d((0.5 * m_Dimensions[0] + Camera.X() * CameraZ) / 256.0 / CameraZ - 0.5, (-0.5 * m_Dimensions[1] + Camera.Y() * CameraZ) / 256.0 / CameraZ - 0.5); glVertex2i(m_Position[0] + m_Dimensions[0], m_Position[1]);
	glEnd();
}

void Canvas::SetupTransform()
{
	glLoadIdentity();
	glTranslated(m_Position.X(), m_Position.Y(), 0);
	if (m_Centered)
	{
		glTranslated(0.5 * m_Dimensions.X(), 0.5 * m_Dimensions.Y(), 0);
	}
	glScaled(CameraZ, CameraZ, CameraZ);
	glTranslated(-Camera.X(), -Camera.Y(), 0);

	glScissor(m_Position.X(), nViewportHeight - (m_Position.Y() + m_Dimensions.Y()), m_Dimensions.X(), m_Dimensions.Y());
	glEnable(GL_SCISSOR_TEST);
}

void Canvas::EndTransform()
{
	glLoadIdentity();

	glDisable(GL_SCISSOR_TEST);
}

const bool Canvas::GlobalIsInsideCanvas(const Vector2d Position) const
{
	return (Position.X() >= m_Position.X()
			&& Position.Y() >= m_Position.Y()
			&& Position.X() <= m_Position.X() + m_Dimensions.X()
			&& Position.Y() <= m_Position.Y() + m_Dimensions.Y());
}

// Precondition: GlobalIsInsideCanvas(Position) is true
const Vector2d Canvas::GlobalToCanvas(const Vector2d Position) const
{
	if (!m_Centered)
	{
		return Vector2d((Position - Vector2d(m_Position.X(), m_Position.Y())) / CameraZ + Camera);
	}
	else
	{
		return Vector2d((Position - Vector2d(m_Position.X(), m_Position.Y()) - Vector2d(m_Dimensions.X(), m_Dimensions.Y()) * 0.5) / CameraZ + Camera);
	}
}

void Canvas::SetDimensions(Vector2n Dimensions)
{
	m_Dimensions = Dimensions;
}
