#include "Main.h"

int nViewportWidth, nViewportHeight;	// The viewport (visible window) size

Project CurrentProject;
Screen * pMainScreen = nullptr;
TypingModule * g_TypingModuleTEST = nullptr;

// Process a window resize event
void GLFWCALL ProcessWindowSize(int nWindowWidth, int nWindowHeight)
{
	printf("WindowResize to %dx%d.\n", nWindowWidth, nWindowHeight);

	// Set the viewport size
	nViewportWidth = nWindowWidth;
	nViewportHeight = nWindowHeight;
	glViewport(0, 0, nViewportWidth, nViewportHeight);

	// Update the projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//gluOrtho2D(nViewportWidth / -2, nViewportWidth / 2, nViewportHeight / 2, nViewportHeight / -2);
	gluOrtho2D(0, nViewportWidth, nViewportHeight, 0);
	glMatrixMode(GL_MODELVIEW);

	pMainScreen->UpdateWindowDimensions(Vector2n(nWindowWidth, nWindowHeight));
}

void InitializeOpenGL()
{
	glEnable(GL_TEXTURE_2D);
	{
		glBindTexture(GL_TEXTURE_2D, 2);
		// http://clkoerner.com/2011/10/13/apple-linen/
		if (GL_TRUE == glfwLoadTexture2D("data/Background.tga", GLFW_ORIGIN_UL_BIT | GLFW_BUILD_MIPMAPS_BIT))
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		}
		else
			throw 1;
	}
	if (!OglUtilsInitFont()) glfwCloseWindow();
	glEnable(GL_CULL_FACE);
	//glClearColor(0.600f, 0.741f, 0.565f, 1);
	glClearColor(1, 1, 1, 1);
}

// Main function
int main(int argc, char * argv[])
{
	glfwInit();
	// Verify the GLFW library and header versions match
	{
		int Major, Minor, Revision;
		glfwGetVersion(&Major, &Minor, &Revision);

		bool Match = (GLFW_VERSION_MAJOR == Major && GLFW_VERSION_MINOR == Minor && GLFW_VERSION_REVISION == Revision);

		if (!Match)
		{
			std::cerr << "Error: GLFW library and header versions do not match." << std::endl;

			return 1;
		}
		else
		{
			std::cout << "Using GLFW v" << Major << "." << Minor << "." << Revision << "." << std::endl;
		}
	}

	GLFWvidmode DesktopMode;
	glfwGetDesktopMode(&DesktopMode);

	//glfwOpenWindowHint(GLFW_FSAA_SAMPLES, 32);

	const bool FULLSCREEN = 0;
	const Vector2n WINDOW_DIMENSIONS(1024, 768);

    if (!FULLSCREEN) {
		glfwOpenWindow(WINDOW_DIMENSIONS.X(), WINDOW_DIMENSIONS.Y(), DesktopMode.RedBits, DesktopMode.GreenBits, DesktopMode.BlueBits, 0, 0, 0, GLFW_WINDOW);
		glfwSetWindowPos((DesktopMode.Width - WINDOW_DIMENSIONS.X()) / 2, (DesktopMode.Height - WINDOW_DIMENSIONS.Y()) / 2);		// Center the window
	} else {
		glfwOpenWindow(DesktopMode.Width, DesktopMode.Height, DesktopMode.RedBits, DesktopMode.GreenBits, DesktopMode.BlueBits, 0, 0, 0, GLFW_FULLSCREEN);
		glfwEnable(GLFW_MOUSE_CURSOR);
	}
	glfwSetWindowTitle("Conception");
	std::stringstream x;
	x << "CPU Count: " << glfwGetNumberOfProcessors()
		<< "\nGL Renderer: " << glGetString(GL_VENDOR) << " " << glGetString(GL_RENDERER) << " v" << glGetString(GL_VERSION)
		<< "\nGLFW_ACCELERATED: " << glfwGetWindowParam(GLFW_ACCELERATED)
		<< "\nGLFW_RED_BITS: " << glfwGetWindowParam(GLFW_RED_BITS)
		<< "\nGLFW_GREEN_BITS: " << glfwGetWindowParam(GLFW_GREEN_BITS)
		<< "\nGLFW_BLUE_BITS: " << glfwGetWindowParam(GLFW_BLUE_BITS)
		<< "\nGLFW_ALPHA_BITS: " << glfwGetWindowParam(GLFW_ALPHA_BITS)
		<< "\nGLFW_DEPTH_BITS: " << glfwGetWindowParam(GLFW_DEPTH_BITS)
		<< "\nGLFW_STENCIL_BITS: " << glfwGetWindowParam(GLFW_STENCIL_BITS)
		<< "\nGLFW_REFRESH_RATE: " << glfwGetWindowParam(GLFW_REFRESH_RATE)
		<< "\nGLFW_FSAA_SAMPLES: " << glfwGetWindowParam(GLFW_FSAA_SAMPLES);
	std::cout << x.str() << std::endl;
	glfwSwapInterval(1);		// Set Vsync

	glfwEnable(GLFW_KEY_REPEAT);

	InputManager InputManager;
	g_InputManager = &InputManager;

	Screen MainScreen(InputManager);
	pMainScreen = &MainScreen;

	// Set up the callbacks
	glfwSetWindowSizeCallback(&ProcessWindowSize);

	// OpenGL settings
	InitializeOpenGL();

	PopulateConcepts();

	// Load program
	CurrentProject.LoadSampleGenProgram();

	// Main loop
	while (glfwGetWindowParam(GLFW_OPENED))
	{
		double CurrentTime = glfwGetTime();
		static double LastTime = CurrentTime;
		double TimePassed = CurrentTime - LastTime;
		LastTime = CurrentTime;
		
		// Clear frame
		glDisable(GL_SCISSOR_TEST);
		glClear(GL_COLOR_BUFFER_BIT);
		glLoadIdentity();

		MainScreen.Render();

		// Display new frame
		glfwSwapBuffers();
		glFinish();

		MainScreen.ProcessTimePassed(TimePassed);

		glfwSleep(0.010);
	}

	// Clean up
	OglUtilsKillFont();
	glfwTerminate();

	return 0;
}
