#include "Main.h"

Screen::Screen(InputManager & InputManager)
	: MainCanvas(Vector2n(50, 50), true),
	  MainCanvas2TEST(Vector2n(800, 100), true),
	  OverlayCanvas(Vector2n(0, 0), false),
	  SystemInputListener(),
	  ControlModuleMapping0(),
	  ControlModuleMapping1(),
	  WidgetManager(),
	  WidgetModule(WidgetManager),
	  UnrealCameraModule(MainCanvas),
	  UnrealCameraModule2TEST(MainCanvas2TEST),
	  TypingModule(),
	  m_InputManager(InputManager)
{
	{
		{ auto Widget = new ButtonWidget(Vector2n(10, 10)); WidgetManager.Add(Widget); Widget->SetOwnerCanvas(OverlayCanvas); }
		{ auto Widget = new ButtonWidget(Vector2n(50, 10)); WidgetManager.Add(Widget); Widget->SetOwnerCanvas(OverlayCanvas); }
		{ auto Widget = new TextFieldWidget(Vector2n(10, 50)); WidgetManager.Add(Widget); Widget->SetOwnerCanvas(OverlayCanvas); }

		{ auto Widget = new ListWidget(Vector2n(-200, -300), CurrentProject.GetStdIncludes()); WidgetManager.Add(Widget); Widget->SetOwnerCanvas(MainCanvas); }
	}

	{
		std::vector<InputManager::InputId> ButtonMappings; ButtonMappings.push_back(InputManager::InputId(1000, GLFW_MOUSE_BUTTON_LEFT));
		std::vector<InputManager::InputId> SliderMappings;
		std::vector<InputManager::InputId> AxisMappings; AxisMappings.push_back(InputManager::InputId(1000, 0)); AxisMappings.push_back(InputManager::InputId(1000, 1));
		std::vector<InputManager::InputId> PositiveConstraints;
		std::vector<InputManager::InputId> NegativeConstraints;
		ControlModuleMapping0.AddMapping(&WidgetModule, 0, ButtonMappings, SliderMappings, AxisMappings, PositiveConstraints, NegativeConstraints);
		WidgetModule.SetOwnerCanvas(OverlayCanvas);
	}
	{
		std::vector<InputManager::InputId> ButtonMappings; ButtonMappings.push_back(InputManager::InputId(1000, GLFW_MOUSE_BUTTON_LEFT));
		std::vector<InputManager::InputId> SliderMappings; SliderMappings.push_back(InputManager::InputId(1000, 0)); SliderMappings.push_back(InputManager::InputId(1000, 1)); SliderMappings.push_back(InputManager::InputId(1000, 2));
		std::vector<InputManager::InputId> AxisMappings; AxisMappings.push_back(InputManager::InputId(1000, 0)); AxisMappings.push_back(InputManager::InputId(1000, 1));
		std::vector<InputManager::InputId> PositiveConstraints;
		std::vector<InputManager::InputId> NegativeConstraints;
		ControlModuleMapping0.AddMapping(&UnrealCameraModule2TEST, 0, ButtonMappings, SliderMappings, AxisMappings, PositiveConstraints, NegativeConstraints);
		UnrealCameraModule2TEST.SetOwnerCanvas(MainCanvas2TEST);
	}
	{
		std::vector<InputManager::InputId> ButtonMappings; ButtonMappings.push_back(InputManager::InputId(1000, GLFW_MOUSE_BUTTON_LEFT));
		std::vector<InputManager::InputId> SliderMappings; SliderMappings.push_back(InputManager::InputId(1000, 0)); SliderMappings.push_back(InputManager::InputId(1000, 1)); SliderMappings.push_back(InputManager::InputId(1000, 2));
		std::vector<InputManager::InputId> AxisMappings; AxisMappings.push_back(InputManager::InputId(1000, 0)); AxisMappings.push_back(InputManager::InputId(1000, 1));
		std::vector<InputManager::InputId> PositiveConstraints;
		std::vector<InputManager::InputId> NegativeConstraints;
		ControlModuleMapping0.AddMapping(&UnrealCameraModule, 0, ButtonMappings, SliderMappings, AxisMappings, PositiveConstraints, NegativeConstraints);
		UnrealCameraModule.SetOwnerCanvas(MainCanvas);
	}
	ControlModuleMapping0.DoneAdding();

	g_TypingModuleTEST = &TypingModule;
	{
		std::vector<InputManager::InputId> ButtonMappings; ButtonMappings.push_back(InputManager::InputId(0, GLFW_KEY_BACKSPACE));
		std::vector<InputManager::InputId> SliderMappings;
		std::vector<InputManager::InputId> AxisMappings; AxisMappings.push_back(InputManager::InputId(1000, 0)); AxisMappings.push_back(InputManager::InputId(1000, 1));
		std::vector<InputManager::InputId> PositiveConstraints;
		std::vector<InputManager::InputId> NegativeConstraints;
		ControlModuleMapping1.AddMapping(&TypingModule, 0, ButtonMappings, SliderMappings, AxisMappings, PositiveConstraints, NegativeConstraints);
		TypingModule.SetOwnerCanvas(OverlayCanvas);
	}
	ControlModuleMapping1.DoneAdding();

	m_InputManager.RegisterListener(&SystemInputListener);
	m_InputManager.RegisterListener(&ControlModuleMapping0);
	m_InputManager.RegisterListener(&ControlModuleMapping1);
}

Screen::~Screen()
{
}

void Screen::UpdateWindowDimensions(Vector2n WindowDimensions)
{
	m_InputManager.UpdateWindowDimensions(WindowDimensions);

	MainCanvas.SetDimensions(Vector2n(WindowDimensions.X() - 200, WindowDimensions.Y() - 200));
	MainCanvas2TEST.SetDimensions(Vector2n(300, 500));
	OverlayCanvas.SetDimensions(WindowDimensions);
}

void Screen::Render()
{
	{
		MainCanvas.RenderBackground();
		
		MainCanvas.SetupTransform();
		{
			CurrentProject.Render();
			
			WidgetManager.Render(MainCanvas);
		}
		MainCanvas.EndTransform();
	}

	{
		MainCanvas2TEST.RenderBackground();
		
		MainCanvas2TEST.SetupTransform();
		{
			CurrentProject.Render();

			WidgetManager.Render(MainCanvas2TEST);
		}
		MainCanvas2TEST.EndTransform();
	}

	{
		OverlayCanvas.SetupTransform();
		{
			WidgetManager.Render(OverlayCanvas);
			
			g_TypingModuleTEST->Render();
		}
		OverlayCanvas.EndTransform();
	}
}

void Screen::ProcessTimePassed(double TimePassed)
{
	ControlModuleMapping0.ProcessTimePassed(TimePassed);
	ControlModuleMapping1.ProcessTimePassed(TimePassed);
}
